import { memoize } from 'underscore';
import hooks from './hooks';

const ua = navigator.userAgent;

/**
 * Check if device is iOS.
 *
 * @returns {boolean}
 */
export const isIOS = memoize(() => {
	return /iphone|ipad|ipod/i.test(ua);
});

/**
 * Check if device is Android.
 *
 * @returns {boolean}
 */
export const isAndroid = memoize(() => {
	return /android/i.test(ua);
});

/**
 * Check if device is mobile device.
 *
 * @returns {boolean}
 */
export const isMobile = memoize(() => {
	return /android|webos|iphone|ipad|ipod|blackberry|iemobile|opera mini/i.test(ua);
});

/**
 * Check if device has touch support.
 *
 * @returns {boolean}
 */
export const isTouch = memoize(() => {
	return 'ontouchstart' in document.documentElement;
});

/**
 * Check whether the browser is active using Page Visibility API.
 *
 * @returns {boolean}
 */
export const isHidden = () => {
	if (typeof document.hidden !== 'undefined') {
		return document.hidden;
	} else {
		return false;
	}
};

/**
 * Trigger event when the browser goes inactive and vice versa.
 */
document.addEventListener('visibilitychange', function() {
	if (isHidden()) {
		hooks.doAction('browser.inactive');
	} else {
		hooks.doAction('browser.active');
	}
});

/**
 * Window unload event handler.
 */
window.addEventListener('beforeunload', e => {
	if (hooks.applyFilters('beforeunload', false)) {
		(e || window.event).returnValue = null;
		return null;
	}
});
